/*
*  Name:      test-1.uesql.cc
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Test for UESQL
*  Date:      $Date: 1999/12/07 10:28:59 $
*  Revision:  $Revision: 1.15 $
*
*  Copyright (C) 1996-1999  Rafael Jesus Alcantara Perez <ralcan@vnet.es>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#include <cstdlib>
#include <iostream>
#include <mpcl/config_processor.hh>
#include <mpcl/string.hh>
#include <mpcl/test.h>


//
//  L O C A L   V A R I A B L E S
//

static uesql::TSqlstateHandler   _tSqlstateHandler;
  

//
//  L O C A L   F U N C T I O N S
//

/**
*  Construct error message from SQL status.
*  @return SQL status message.
*/
static std::string _GetErrorMessage (void)
{

  return ( std::string (_tSqlstateHandler.getClass())    +
           ": "                                          +
           std::string (_tSqlstateHandler.getSubclass()) +
           " (" + uesql::SQLMESSAGE + ")"                );

}  // _GetErrorMessage()


int TestCursors (void)
{

  std::string   yVar0;
  std::string   yVar1;
  std::string   yVar2;
  bool          gEmpty       = true;
  std::string   yAttribName  = "WORK_CODE";
  size_t        zRecordCount = 0;

  TEST_INIT ("test for cursors");

  extern "UESQL::default" 
  {
    whenever sqlerror  goto error;
    whenever not found goto loop_end;
    declare MYCURSOR cursor for select NAME, HOURS from WORKERS where NAME = 'nobody';
    open MYCURSOR;
  }
  while ( true )
  {
    extern "UESQL::default"
    {
      fetch from MYCURSOR into :yVar0, :yVar1;
    }
    gEmpty = false;
  }

loop_end:
  extern "UESQL::default" 
  {
    close MYCURSOR;
  }
  if ( gEmpty )
  {
    TEST_PASS;
  }
  else
  {
    TEST_FAIL;
  }

  extern "UESQL::default"
  {
    --
    --  WHENEVER
    --
    whenever sqlerror  goto error;
    whenever not found goto loop_end_2;

    --
    --  DECLARE CURSOR/OPEN/WHENEVER
    --
    declare MYCURSOR cursor for select :yAttribName, NAME, HOURS from WORKERS;
    open MYCURSOR;
  }
    
  if ( _ptTargetFile )
  {
    fprintf (_ptTargetFile, "%s\n", "WORK_CODE NAME                                     HOURS");
    fprintf (_ptTargetFile, "%s\n", "========= ======================================== =====");
  }
  while ( true )
  {
    extern "UESQL::default"
    {
      fetch from MYCURSOR into :yVar0, :yVar1, :yVar2;
    }
    if ( _ptTargetFile )
    {
      fprintf (_ptTargetFile, "%9s %40s %5s\n", yVar0.c_str(), yVar1.c_str(), yVar2.c_str());
    }
    ++zRecordCount;
  }

loop_end_2:
  extern "UESQL::default"
  {
    --
    --  CLOSE CURSOR
    --
    close MYCURSOR;
  }
  goto end;

error:
  cerr << "SQLSTATE="    << uesql::SQLSTATE    << endl;
  cerr << "SQLCODE="     << uesql::SQLCODE     << endl;
  cerr << "DESCRIPTION=" << _GetErrorMessage() << endl;
  TEST_FAIL_WITH_CODE (uesql::SQLCODE);  

end:
  TEST_MEMORY_STATUS;
  TEST_RETURN_CODE;
  
}  // TestCursors()


int TestSchemas ( const std::string& rkySOURCE_HOSTNAME ,
                  const std::string& rkyUSER            )
{

  TEST_INIT ("test for schemas");

  mpcl::xstring   yNewSchemaServer = mpcl::Format ("%s/TEST_1_SCHEMA", rkySOURCE_HOSTNAME.c_str());
  
  extern "UESQL::default" 
  {
    whenever sqlerror goto error;
    --
    --  CREATE SCHEMA
    --
    create schema TEST_1_SCHEMA
      create table TABLE_1
      (
        CODE         character (3) not null ,
        WORKER_COUNT integer
      )
      create table TABLE_2
      (
        WORK_CODE character (3)  not null ,
        NAME      character (40) not null ,
        HOURS     decimal   (5)
      );
    connect to :uesql::Quote (yNewSchemaServer) as 'myconnection' user :uesql::Quote (rkyUSER);
    drop table TABLE_1 cascade;
    disconnect current;
    set connection default;
    drop schema TEST_1_SCHEMA cascade;
  }
  TEST_PASS;
  goto end;

error:
  cerr << "SQLSTATE="    << uesql::SQLSTATE    << endl;
  cerr << "SQLCODE="     << uesql::SQLCODE     << endl;
  cerr << "DESCRIPTION=" << _GetErrorMessage() << endl;
  TEST_FAIL_WITH_CODE (uesql::SQLCODE);  

end:  
  TEST_MEMORY_STATUS;
  TEST_RETURN_CODE;
  
}  // TestSchemas()


/**
*  Main.
*
*  Parameters:
*
*    {--hostname, -o}=HOSTNAME
*    {--username, -u}=USERNAME
*    {--password, -p}=PASSWORD
*/
int main (int argc, const char* argv[])
{

  mpcl::TConfigProcessor   tConfig;

  TEST_INIT ("test-base-1");
  TEST_START_WATCH;

  std::string   yVar0;
  std::string   yVar1;
  std::string   yVar2;
  std::string   yCursorName;
  std::string   yUserName;
  std::string   yHostName;
  std::string*  pyUserName = &yUserName;
  std::string*  pyHostName = &yHostName;
  std::string   yCodeValue = "002";

  try
  {
    //
    //  Define string constants names for options.
    // 
    tConfig.addOption ("hostname", "o", "localhost");
    tConfig.addOption ("username", "u", getenv ("USER"));
    tConfig.addOption ("password", "p", "");
    tConfig.setCommandLineParameters (argc, argv);
    tConfig.processOptions();
    
    yHostName = tConfig ["hostname"].value();
    if ( tConfig ["password"].value().empty() )
    {
      yUserName = tConfig ["username"].value();
    }
    else
    {
      yUserName = mpcl::Format ( "%s/%s"                              ,
                                 tConfig ["username"].value().c_str() ,
                                 tConfig ["password"].value().c_str() );
    }
  }
  catch (const mpcl::TBaseException& rktSOURCE_EXCEPTION)
  {
    cerr << rktSOURCE_EXCEPTION;
    TEST_FAIL_WITH_CODE (3);
  }

  TEST_WRITE_INFO ("HOSTNAME = %s", yHostName.c_str());
  TEST_WRITE_INFO ("USER     = %s", yUserName.c_str());
  if ( !yHostName.empty() && !yUserName.empty() )
  {
    extern "UESQL::default" 
    {
      --
      --  WHENEVER
      --
      whenever sqlerror goto error;
      whenever not found goto not_found;
      
      --
      --  CONNECT TO
      --
      connect to :uesql::Quote (yHostName) user :uesql::Quote (yUserName);

      --
      --  CREATE TABLE
      --
      create table JOBS
      (
        CODE         character (3) not null ,
        WORKER_COUNT integer
      );

      create table WORKERS
      (
        WORK_CODE character (3)  not null ,
        NAME      character (40) not null ,
        HOURS     decimal (5)
      );

      --
      --  ALTER TABLE
      --
      alter table WORKERS add column STATE character (40);
      alter table WORKERS add column PHONE_NUMBER integer;

      --
      --  DELETE
      --
      delete from JOBS where CODE = :uesql::Quote (yCodeValue);
      delete from JOBS where CODE = '001';
      delete from WORKERS;

      --
      --  INSERT
      --
      insert into JOBS values ('001', 100);
      insert into JOBS values ('002', 250);
      insert into WORKERS values ('001', 'myname',   40, 'Andalucia',   5550000);
      insert into WORKERS values ('001', 'yourname', 35, 'Extremadura', 5550001);
      insert into WORKERS values ('001', 'hisname',  25, 'Catalunya',   5550002);
      insert into WORKERS values ('002', 'john',     35, 'New England', 5550003);
      insert into WORKERS values ('002', 'james',    35, 'Washington',  5550004);
      insert into WORKERS select WORK_CODE, NAME, HOURS, STATE, PHONE_NUMBER from WORKERS where NAME = 'myname';
      insert into WORKERS select WORK_CODE, NAME, HOURS, STATE, PHONE_NUMBER from WORKERS;

      --
      --  WHENEVER
      --
      whenever sqlerror goto cardinality_check;

      --
      --  SELECT for a single row.
      --
      --    Cardinality violation, so ( SQLSTATE == '21000' ).
      --
      select HOURS into :yVar1 from WORKERS, JOBS where WORK_CODE = CODE;
    }
    TEST_FAIL;

cardinality_check:
    TEST_STRINGS (uesql::SQLSTATE.c_str(), "21000");
    yCursorName = "MYCURSOR";
    extern "UESQL::default" 
    {
      --
      --  WHENEVER
      --
      whenever sqlerror goto error;

      --
      --  SELECT for a single row.
      --
      select count (*) into :yVar2 from WORKERS where name = 'myname';

      --
      --  UPDATE
      --
      update WORKERS
      set PHONE_NUMBER = WORKERS.PHONE_NUMBER
      where WORKERS.NAME = 'myname' and WORKERS.NAME = NAME;
    }
    TestCursors();
    TestSchemas (yHostName, yUserName);
    extern "UESQL::default" 
    {
      --
      --  DECLARE CURSOR/OPEN/FETCH/CLOSE
      --
      declare MYCURSOR cursor for select CODE from JOBS where CODE = '001';
      open MYCURSOR;
      fetch from :yCursorName into :yVar0;
      close MYCURSOR;
    }
    TEST_STRINGS (uesql::SQLSTATE.c_str(), "00000");
    TEST_STRINGS (yVar2.c_str(),           "4");
    TEST_STRINGS (yVar0.c_str(),           "001");
    
    extern "UESQL::default" 
    {
      --
      --  REVOKE
      --
      revoke all privileges on JOBS from public cascade;
      revoke all privileges on WORKERS from public restrict;

      --
      --  GRANT
      --
      grant select on JOBS    to public;
      grant select on WORKERS to public;
      grant delete on WORKERS to public;

      --
      --  CREATE VIEW
      --
      create view VIEW1 as select * from JOBS;

      --
      --  DROP VIEW
      --
      drop view VIEW1 cascade;

      --
      --  DROP TABLE
      --
      drop table JOBS cascade;

      --
      --  DISCONNECT
      --
      disconnect current;
    }

    extern "UESQL::default"
    {
      --
      --  CONNECT TO
      --
      connect to :uesql::Quote (*pyHostName) as 'myconnection' user :uesql::Quote (*pyUserName);

      --
      --  DROP TABLE
      --
      drop table WORKERS cascade;
    }
    goto end;

  error:
    cerr << "SQLSTATE="    << uesql::SQLSTATE    << endl;
    cerr << "SQLCODE="     << uesql::SQLCODE     << endl;
    cerr << "DESCRIPTION=" << _GetErrorMessage() << endl;
    TEST_FAIL_WITH_CODE (uesql::SQLCODE);
    goto end;

  not_found:
    cerr << "Search failed\n";
    TEST_FAIL_WITH_CODE (2);
    goto end;
  }

end:
  TEST_STOP_WATCH;
  TEST_DISPLAY_WATCH;
  TEST_MEMORY_STATUS;
  TEST_RETURN_CODE;

}  // main()
